import numpy as np


def project_pcl_to_image(pcl, t_camera_radar, camera_projection_matrix, image_shape):
    location = np.hstack((pcl[['x', 'y', 'z']],
                          np.ones((pcl.shape[0], 1))))
    # print(location)

    radar_points_camera_frame = t_camera_radar.dot(location.T).T

    point_depth = radar_points_camera_frame[:, 2]

    uvs = project_3d_to_2d(radar_points_camera_frame, camera_projection_matrix)
    power = pcl['rcs']

    return uvs, point_depth, power, pcl


def project_3d_to_2d(points: np.ndarray, projection_matrix: np.ndarray):
    """
This function projects the input 3d ndarray to a 2d ndarray, given a projection matrix.
    :param points: Homogenous points to be projected.
    :param projection_matrix: 4x4 projection matrix.
    :return: 2d ndarray of the projected points.
    """
    if points.shape[-1] != 4:
        raise ValueError(f"{points.shape[-1]} must be 4!")

    uvw = projection_matrix.dot(points.T)
    uvw /= uvw[2]
    uvs = uvw[:2].T
    uvs = np.round(uvs).astype(np.int)

    return uvs


def canvas_crop(points, image_size, pcl, points_depth=None):
    """
This function filters points that lie outside a given frame size.
    :param power:
    :param points: Input points to be filtered.
    :param image_siz e: Size of the frame.
    :param points_depth: Filters also depths smaller than 0.
    :return: Filtered points.
    """
    idx = points[:, 0] > 0
    idx = np.logical_and(idx, points[:, 0] < image_size[1])
    idx = np.logical_and(idx, points[:, 1] > 0)
    idx = np.logical_and(idx, points[:, 1] < image_size[0])
    idx = np.logical_and(idx, pcl[:, 3] > 0)

    # tmp
    # idx = np.logical_and(idx, pcl[:, 1] < -7)
    if points_depth is not None:
        idx = np.logical_and(idx, points_depth > 0)

    return idx



import os
import pandas as pd
import numpy as np


def get_t_camera_radar(dataset_path, frame):
    with open(os.path.join(dataset_path, 'calib', f"{frame}.txt"), "r") as f:
        lines = f.readlines()
        matrix = np.array(lines[0].strip().split(' ')[1:], dtype=np.float32).reshape(4, 4)

    return matrix


def get_camera_projection_matrix(dataset_path, frame):
    with open(os.path.join(dataset_path, 'calib', f"{frame}.txt"), "r") as f:
        lines = f.readlines()
        matrix = np.array(lines[1].strip().split(' ')[1:], dtype=np.float32).reshape(3, 4)

    return matrix


def project_pcl_to_image(pcl, t_camera_radar, camera_projection_matrix):
    location = np.hstack((pcl[['x', 'y', 'z']],
                          np.ones((pcl.shape[0], 1))))
    print('location', location)
    radar_points_camera_frame = t_camera_radar.dot(location.T).T
    print('radar_points_camera_frame', radar_points_camera_frame)

    return project_3d_to_2d(radar_points_camera_frame, camera_projection_matrix)


def project_3d_to_2d(points: np.ndarray, projection_matrix: np.ndarray):
    if points.shape[-1] != 4:
        raise ValueError(f"{points.shape[-1]} must be 4!")

    uvw = projection_matrix.dot(points.T)
    print('uvw', uvw)
    uvw /= uvw[2]



    uvs = uvw[:2].T
    print('uvs', uvs)
    uvs = np.round(uvs).astype(np.int32)

    return uvs


if __name__ == '__main__':
    dataset_path = "/data/waterscenes/all"
    frame = '1664246698.44130'

    # Get radar data
    radar = pd.read_csv(os.path.join(dataset_path, 'radar', f'{frame}.csv'))
    print(radar[['x', 'y', 'z', 'u', 'v']])

    # Get intrinsic and extrinsic
    t_camera_radar = get_t_camera_radar(dataset_path, frame)
    print(t_camera_radar)
    camera_projection_matrix = get_camera_projection_matrix(dataset_path, frame)
    print(camera_projection_matrix)

    # Transform pcl to uvs
    uvs = project_pcl_to_image(radar, t_camera_radar, camera_projection_matrix)
    print(uvs)



from matplotlib import pyplot as plt
import os
import numpy as np
import pandas as pd


class DataLoader:
    def __init__(self, dataset_path, frame):
        self.dataset_path = dataset_path
        self.frame = frame

    @property
    def image(self):
        return self.get_image()

    @property
    def radar_data(self):
        return self.get_radar_data()

    @property
    def radar_3_frames_data(self):
        return self.get_radar_3_frames_data()

    @property
    def label_data(self):
        return self.get_label_data()

    @property
    def t_camera_radar(self):
        return self.get_t_camera_radar()

    @property
    def camera_projection_matrix(self):
        return self.get_camera_projection_matrix()

    def get_label_data(self):
        labels = []
        f = open(os.path.join(self.dataset_path, 'detection/yolo', f'{self.frame}.txt'))
        for line in f.readlines():
            width = 1920
            height = 1080
            label_class, x, y, w, h = line.split()
            label_class = int(label_class)
            x = float(x)
            y = float(y)
            w = float(w)
            h = float(h)
            xmin = int(width * (x - 0.5 * w))
            ymin = int(height * (y - 0.5 * h))
            xmax = int(width * (x + 0.5 * w))
            ymax = int(height * (y + 0.5 * h))

            labels.append({'class': label_class,
                           'xmin': xmin,
                           'ymin': ymin,
                           'xmax': xmax,
                           'ymax': ymax}
                          )
        return labels

    def get_image(self):
        return plt.imread(os.path.join(self.dataset_path, 'image', f'{self.frame}.jpg'))

    def get_radar_data(self):
        return pd.read_csv(os.path.join(self.dataset_path, 'radar', f'{self.frame}.csv'))

    def get_radar_3_frames_data(self):
        return pd.read_csv(os.path.join(self.dataset_path, 'radar_3_frames', f'{self.frame}.csv'))

    def get_t_camera_radar(self):
        with open(os.path.join(self.dataset_path, 'calib', f"{self.frame}.txt"), "r") as f:
            lines = f.readlines()
            matrix = np.array(lines[0].strip().split(' ')[1:], dtype=np.float32).reshape(4, 4)

        return matrix

    def get_camera_projection_matrix(self):
        with open(os.path.join(self.dataset_path, 'calib', f"{self.frame}.txt"), "r") as f:
            lines = f.readlines()
            matrix = np.array(lines[1].strip().split(' ')[1:], dtype=np.float32).reshape(3, 4)

        return matrix



import setuptools

requirements = ['numpy']  # 自定义工具中需要的依赖包

setuptools.setup(
    name="WaterScenes",  # 自定义工具包的名字
    version="1.0.2",  # 版本号
    author="Shanliang Yao",  # 作者名字
    author_email="shanliang.yao19@gmail.com",  # 作者邮箱
    description="Toolkit for WaterScenes",  # 自定义工具包的简介
    license='MIT-0',  # 许可协议
    url="https://github.com/waterscenes",  # 项目开源地址
    packages=setuptools.find_packages(),  # 自动发现自定义工具包中的所有包和子包
    install_requires=requirements,  # 安装自定义工具包需要依赖的包
    python_requires='>=3.8'  # 自定义工具包对于python版本的要求
)



from WaterScenes.DataLoader import DataLoader
from matplotlib import pyplot as plt
from WaterScenes.Transformation import project_pcl_to_image
from WaterScenes.Config import label_color, label_code
import os
from mpl_toolkits.mplot3d import Axes3D
import numpy as np


class WaterScenes:

    def __init__(self, dataset_path=None, frame=''):
        self.dataset_path = dataset_path
        self.dataloader = DataLoader(self.dataset_path, frame)

    def visualization(self, show_labels=False,
                      show_radar_label=False,
                      show_radar=False,
                      plot_figure=True,
                      save_figure=False):
        fig = plt.figure(figsize=(19.2, 10.8))
        plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
        plt.margins(0, 0)
        plt.clf()
        fig.set_dpi(100)

        if show_labels:
            self.plot_gt_labels()

        if show_radar:
            self.plot_radar_pcl()
            # self.plot_radar_pcl_from_calibration()

        if show_radar_label:
            self.plot_radar_label()

        plt.imshow(self.dataloader.image, alpha=1)
        # plt.colorbar()
        plt.axis('off')

        if save_figure:
            visualization_path = 'visualization'
            if not os.path.exists(visualization_path):
                os.mkdir(visualization_path)
            plt.savefig(visualization_path + f'/{self.dataloader.frame}.png')

        if plot_figure:
            plt.show()

        plt.close(fig)

        return

    def plot_boxes(self, locations, color):
        x1 = locations[0]
        x2 = locations[2]
        y1 = locations[1]
        y2 = locations[3]

        plt.plot([x1, x1], [y1, y2], color=color)
        plt.plot([x1, x2], [y1, y1], color=color)
        plt.plot([x2, x2], [y1, y2], color=color)
        plt.plot([x1, x2], [y2, y2], color=color)

    def plot_gt_labels(self):
        gt_labels = self.dataloader.get_label_data()
        for gt_label in gt_labels:
            self.plot_boxes([gt_label['xmin'], gt_label['ymax'], gt_label['xmax'], gt_label['ymin']],
                            label_color[gt_label['class']])

    def plot_radar_pcl(self):

        radar = self.dataloader.radar_data
        # radar = radar.append({'u': -1, 'v': -1, 'range': 100, 'power': 1}, ignore_index=True)
        power = radar['power']
        points_depth = radar['range']

        plt.scatter(radar['u'], radar['v'], c=-points_depth, s=power * power, alpha=0.8, cmap='jet')

    def plot_radar_pcl_from_calibration(self):
        uvs, points_depth, power, _ = project_pcl_to_image(self.dataloader.radar_data,
                                                           self.dataloader.t_camera_radar,
                                                           self.dataloader.camera_projection_matrix,
                                                           self.dataloader.image.shape)

        plt.scatter(uvs[:, 0], uvs[:, 1], c=-points_depth, alpha=0.8, cmap='jet')

    def plot_radar_label(self):
        radar = self.dataloader.radar_data
        idx = radar['label'] != -1
        uvs = radar[idx]
        plt.scatter(uvs['u'], uvs['v'], alpha=0.8, marker="*")

    def visualization3D(self, frames=1, coordinate='Cartesian'):

        if frames == 1:
            radar = self.dataloader.radar_data
            labels = np.array(radar[['label']]).squeeze(1)
            labels_3D = ['r' if c > -1 else 'b' for c in labels]

        if frames == 3:
            radar = self.dataloader.radar_3_frames_data

            # No labels for radar_3_frames
            labels_3D = ['b']

        fig = plt.figure(dpi=200)
        # ax = Axes3D(fig)
        ax = fig.add_subplot(projection='3d')

        if coordinate == 'Cartesian':
            ax.scatter(radar[['x']], radar[['y']], radar[['z']],
                       # cmap='spectral',
                       s=radar[['power']],
                       linewidth=0,
                       alpha=1,
                       marker=".",
                       c=labels_3D)
            ax.set_xlabel('X', fontsize=8, fontweight='bold')
            ax.set_ylabel('Y', fontsize=8, fontweight='bold')
            ax.set_zlabel('Z', fontsize=8, fontweight='bold')

        if coordinate == 'World':
            ax.scatter(radar[['x']], radar[['z']], -radar[['y']],
                       # cmap='spectral',
                       s=radar[['power']],
                       linewidth=0,
                       alpha=1,
                       marker=".",
                       c=labels_3D)
            ax.set_xlabel('X', fontsize=8, fontweight='bold')
            ax.set_ylabel('Z', fontsize=8, fontweight='bold')
            ax.set_zlabel('Y', fontsize=8, fontweight='bold')

        plt.show()
